package com.lutao.uav.mqtt;

import cn.hutool.json.JSONUtil;
import com.alibaba.fastjson.JSONObject;

import com.baomidou.mybatisplus.core.toolkit.StringUtils;
import com.lutao.uav.domain.entity.NorthSeaGasPoint;
import com.lutao.uav.service.NorthSeaGasPointService;
import lombok.extern.slf4j.Slf4j;
import org.eclipse.paho.client.mqttv3.*;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.data.geo.*;
import org.springframework.data.redis.connection.RedisGeoCommands;
import org.springframework.data.redis.core.BoundGeoOperations;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.kafka.core.KafkaTemplate;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Component;

import javax.annotation.PostConstruct;
import javax.annotation.PreDestroy;
import javax.annotation.Resource;
import java.time.Instant;
import java.time.LocalDateTime;
import java.time.ZoneId;
import java.time.ZonedDateTime;
import java.time.format.DateTimeFormatter;
import java.util.HashMap;
import java.util.List;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;

@Component
@Slf4j
public class MqttConfig {

    @Value("${north-sea-mqtt.broker-url}")
    private String brokerUrl;

    @Value("${north-sea-mqtt.username}")
    private String username;

    @Value("${north-sea-mqtt.password}")
    private String password;

    @Value("${north-sea-mqtt.topic}")
    private String topic;

    @Resource(name = "redisTemplateLettuce")
    private RedisTemplate redisTemplate;

    @Resource
    private NorthSeaGasPointService northSeaGasPointService;

    @Resource
    private KafkaTemplate kafkaTemplate;

    private MqttClient client;
    private ExecutorService executorService;

    @PostConstruct
    public void init() {
        initPointData();
        executorService = Executors.newSingleThreadExecutor();
        connectWithReconnect();
        test();
    }

    //初始化点位数据
    public void initPointData() {
        log.info("-------------------初始化北海点位数据-------------------");
        List<NorthSeaGasPoint> northSeaGasPoints = northSeaGasPointService.list();
        HashMap<Object, Object> map = new HashMap<>();
        BoundGeoOperations northSeaPoint = redisTemplate.boundGeoOps("north_sea_point");
        northSeaGasPoints.forEach(northSeaGasPoint -> {
            String jsonStr = JSONUtil.toJsonStr(northSeaGasPoint);
            map.put(jsonStr, new Point(Double.parseDouble(northSeaGasPoint.getLongitude()), Double.parseDouble(northSeaGasPoint.getLatitude())));
        });
        northSeaPoint.add(map);
        log.info("-------------------初始化成功-------------------");
    }

    private void connectWithReconnect() {
        new Thread(() -> {
            while (!Thread.currentThread().isInterrupted()) {
                try {
                    MqttConnectOptions options = new MqttConnectOptions();
                    options.setCleanSession(true);
                    options.setKeepAliveInterval(10);
                    options.setConnectionTimeout(30);
                    options.setUserName(username);
                    options.setPassword(password.toCharArray());

                    client = new MqttClient(brokerUrl, MqttClient.generateClientId());
                    client.setCallback(new MqttCallback() {
                        @Override
                        public void connectionLost(Throwable cause) {
                            log.info("MQTT连接丢失，尝试重连:{}", cause.getMessage());
                            // 重置客户端实例，以便重新连接
                            client = null;
                            connectWithReconnect(); // 递归调用重连方法
                        }

                        @Override
                        public void messageArrived(String topic, MqttMessage message) throws Exception {
                            String content = new String(message.getPayload());
                            log.info("接收到主题:{}的消息{}", topic, content);
                            if (JSONUtil.isJson(content)) {
                                //handleJsonMessage(JSONObject.parseObject(content));
                                JSONObject jsonObject = JSONObject.parseObject(content);
                                double longitude = jsonObject.getDoubleValue("longitude");
                                double latitude = jsonObject.getDoubleValue("latitude");
                                try {
                                    Point point = new Point(longitude, latitude);
                                    Distance distance = new Distance(1000000, RedisGeoCommands.DistanceUnit.KILOMETERS);
                                    Circle circle = new Circle(point, distance);

                                    RedisGeoCommands.GeoRadiusCommandArgs args = RedisGeoCommands.GeoRadiusCommandArgs.newGeoRadiusArgs()
                                        .sortAscending()
                                        .includeCoordinates()
                                        .includeDistance()
                                        .limit(1);

                                    GeoResults<RedisGeoCommands.GeoLocation<String>> outlets = redisTemplate.boundGeoOps("north_sea_point").radius(circle, args);
                                    if (outlets != null) {
                                        for (GeoResult<RedisGeoCommands.GeoLocation<String>> outlet : outlets) {
                                            log.info("需要传送的消息: {}", outlet.getContent().getName());
                                            // TODO: 2024/6/21 send to kafka

                                            JSONObject sendMessage = JSONObject.parseObject(outlet.getContent().getName());

                                            /**
                                             * {
                                             * 	"enterpriseId":"140170010",
                                             * "equipCode":"141110055001G0001",
                                             * "surveyInstrumentCode":"141110055001G0001YL001",
                                             * "monitorType":"6",
                                             * "monitorUnit":"mm",
                                             * "realtimeValue":"1.57",
                                             * "collectTime":"2024-04-12 18:21:11"
                                             * }
                                             */

                                            JSONObject kafkaSendMessage = new JSONObject();
                                            kafkaSendMessage.put("enterpriseId", sendMessage.get("companyName"));
                                            kafkaSendMessage.put("equipCode", sendMessage.get("pointCode"));
                                            kafkaSendMessage.put("surveyInstrumentCode", sendMessage.get("pointCode2"));
                                            kafkaSendMessage.put("monitorUnit", sendMessage.get("unit"));
                                            String utcTimeStr = jsonObject.getString("utcTime");
                                            DateTimeFormatter formatter = DateTimeFormatter.ofPattern("yyyy-MM-dd-HH-mm-ss");
                                            LocalDateTime localDateTime = LocalDateTime.parse(utcTimeStr, formatter);

                                            // 将LocalDateTime转换为Instant
                                            Instant instant = localDateTime.atZone(ZoneId.of("UTC")).toInstant();

                                            // 转换到东八区
                                            ZonedDateTime shanghaiTime = instant.atZone(ZoneId.of("Asia/Shanghai"));

                                            // 格式化为字符串
                                            String formattedTime = shanghaiTime.format(DateTimeFormatter.ISO_LOCAL_DATE_TIME);

                                            // 定义输入和输出的日期时间格式
                                            DateTimeFormatter inputFormat = DateTimeFormatter.ofPattern("yyyy-MM-dd'T'HH:mm:ss");
                                            DateTimeFormatter outputFormat = DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss");

                                            // 解析原始字符串到LocalDateTime对象
                                            LocalDateTime dateTime = LocalDateTime.parse(formattedTime, inputFormat);

                                            // 将LocalDateTime对象格式化为新的字符串格式
                                            String formattedDateTime = dateTime.format(outputFormat);

                                            kafkaSendMessage.put("collectTime", formattedDateTime);

                                            // kafkaSendMessage.put("realtimeValue", sendMessage.get("unit"));

                                            //NH3/NO2/CH4/H2S/SO2/HCL/CO
                                            //3 有毒气体  4 可燃气体
                                            String pointType = sendMessage.getString("pointType");
                                            if ("氨气".equals(pointType)){ //可燃 同时有毒
                                                String NH3 = jsonObject.getJSONObject("airData").getString("NH3(ppm)");
                                                kafkaSendMessage.put("monitorType", "4");
                                                kafkaSendMessage.put("realtimeValue", NH3);

                                                // TODO: 2024/6/24 kafka send 两次
                                                try {
                                                    kafkaTemplate.send("majorHazard", kafkaSendMessage.toJSONString());
                                                    log.info("发送了 majorHazard 消息,气体{},内容{}",pointType,kafkaSendMessage);
                                                    kafkaSendMessage.put("monitorType", "3");
                                                    kafkaTemplate.send("majorHazard", kafkaSendMessage.toJSONString());
                                                    log.info("发送了 majorHazard 消息,气体{},内容{}",pointType,kafkaSendMessage);
                                                }catch (Exception e){
                                                    log.error("发送消息失败,气体{},内容{}",pointType,kafkaSendMessage);
                                                }
                                            }

                                            if ("甲烷".equals(pointType)){
                                                String CH4 = jsonObject.getJSONObject("airData").getString("CxHy(%)");
                                                kafkaSendMessage.put("monitorType", "4");
                                                kafkaSendMessage.put("realtimeValue", CH4);

                                                // TODO: 2024/6/24 kafka send
                                               kafkaTemplate.send("majorHazard", kafkaSendMessage.toJSONString());

                                                log.info("发送了 majorHazard 消息,气体{},内容{}",pointType,kafkaSendMessage);
                                            }

                                            if ("硫化氢".equals(pointType)){ //可燃 同时有毒
                                                String H2S = jsonObject.getJSONObject("airData").getString("H2S(ppm)");
                                                kafkaSendMessage.put("monitorType", "4");
                                                kafkaSendMessage.put("realtimeValue", H2S);

                                                // TODO: 2024/6/24 kafka send 两次
                                               kafkaTemplate.send("majorHazard", kafkaSendMessage.toJSONString());
                                                log.info("发送了 majorHazard 消息,气体{},内容{}",pointType,kafkaSendMessage);
                                                kafkaSendMessage.put("monitorType", "3");
                                               kafkaTemplate.send("majorHazard", kafkaSendMessage.toJSONString());
                                                log.info("发送了 majorHazard 消息,气体{},内容{}",pointType,kafkaSendMessage);
                                            }

                                            if ("一氧化碳".equals(pointType)){ //可燃 同时有毒
                                                String CO = jsonObject.getJSONObject("airData").getString("CO(ppm)");
                                                kafkaSendMessage.put("monitorType", "4");
                                                kafkaSendMessage.put("realtimeValue", CO);

                                                // TODO: 2024/6/24 kafka send 两次
                                               kafkaTemplate.send("majorHazard", kafkaSendMessage.toJSONString());
                                                log.info("发送了 majorHazard 消息,气体{},内容{}",pointType,kafkaSendMessage);
                                                kafkaSendMessage.put("monitorType", "3");
                                               kafkaTemplate.send("majorHazard", kafkaSendMessage.toJSONString());
                                                log.info("发送了 majorHazard 消息,气体{},内容{}",pointType,kafkaSendMessage);
                                            }

                                            if ("二氧化氮".equals(pointType)){
                                                String NO2 = jsonObject.getJSONObject("airData").getString("NO2(ppm)");
                                                kafkaSendMessage.put("monitorType", "3");
                                                kafkaSendMessage.put("realtimeValue", NO2);

                                                // TODO: 2024/6/24 kafka send
                                               kafkaTemplate.send("majorHazard", kafkaSendMessage.toJSONString());
                                                log.info("发送了 majorHazard 消息,气体{},内容{}",pointType,kafkaSendMessage);
                                            }

                                            if ("二氧化硫".equals(pointType)){
                                                String SO2 = jsonObject.getJSONObject("airData").getString("SO2(ppm)");
                                                kafkaSendMessage.put("monitorType", "3");
                                                kafkaSendMessage.put("realtimeValue", SO2);

                                                // TODO: 2024/6/24 kafka  send
                                               kafkaTemplate.send("majorHazard", kafkaSendMessage.toJSONString());
                                                log.info("发送了 majorHazard 消息,气体{},内容{}",pointType,kafkaSendMessage);
                                            }

                                            if ("氯化氢".equals(pointType)){
                                                String HCL = jsonObject.getJSONObject("airData").getString("HCL(ppm)");
                                                kafkaSendMessage.put("monitorType", "3");
                                                kafkaSendMessage.put("realtimeValue", HCL);

                                                // TODO: 2024/6/24 kafka  send
                                               kafkaTemplate.send("majorHazard", kafkaSendMessage.toJSONString());
                                                log.info("发送了 majorHazard 消息,气体{},内容{}",pointType,kafkaSendMessage);
                                            }



                                        }
                                    }


                                } catch (Exception e) {
                                    e.printStackTrace();
                                    log.error("MqttClient receive error:{}", e.getMessage());
                                }
                            }
                        }

                        @Override
                        public void deliveryComplete(IMqttDeliveryToken token) {
                            // 消息已送达
                        }
                    });

                    client.connect(options);
                    client.subscribe(topic);
                    log.info("-------------------MQTT客户端已连接并订阅主题:{}-------------------", topic);
                    break; // 连接成功，退出循环
                } catch (MqttException e) {
                    //System.out.println("MQTT连接失败，3秒后重试: " + e.getMessage());
                    log.info("MQTT连接失败，3秒后重试:{}", e.getMessage());
                    try {
                        Thread.sleep(3000); // 等待3秒
                    } catch (InterruptedException e1) {
                        Thread.currentThread().interrupt(); // 重新设置中断状态
                        return;
                    }
                }
            }
        }).start();
    }

    public void test(){
        Point point = new Point(0, 0);
        Distance distance = new Distance(1000000, RedisGeoCommands.DistanceUnit.KILOMETERS);
        Circle circle = new Circle(point, distance);

        RedisGeoCommands.GeoRadiusCommandArgs args = RedisGeoCommands.GeoRadiusCommandArgs.newGeoRadiusArgs()
                .sortAscending()
                .includeCoordinates()
                .includeDistance()
                .limit(1);

        GeoResults<RedisGeoCommands.GeoLocation<String>> outlets = redisTemplate.boundGeoOps("north_sea_point").radius(circle, args);
        outlets.forEach(outlet -> {
            log.info("需要传送的消息: {}", outlet.getContent().getName());
        });
    }

    //@Scheduled(fixedRate = 2000)
    public void testsend() {
        kafkaTemplate.send("majorHazard", "测试消息");
        log.info("发送了 majorHazard 消息");
    }

    @PreDestroy
    public void destroy() {
        if (client != null && client.isConnected()) {
            try {
                client.disconnect();
            } catch (MqttException e) {
                e.printStackTrace();
            }
        }
        if (executorService != null) {
            executorService.shutdownNow();
        }
    }
}
